#!/usr/bin/env python
#======================================================================
# FILE: Store.py
# CREATOR: eric
#
# (C) COPYRIGHT 2001, Eric Busboom <eric@civicknowledge.com>
# (C) COPYRIGHT 2001, Patrick Lewis <plewis@inetarena.com>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#======================================================================

from LibicalWrap import *
from Error import LibicalError
from Component import Component, CloneComponent
from Gauge import Gauge

class Store:
    """
    Base class for several component storage methods
    """

    class AddFailedError(LibicalError):
        "Failed to add a property to the file store"

    class ConstructorFailedError(LibicalError):
        "Failed to create a Store "

    def __init__(self):
        pass

    def path(self):
        pass

    def mark(self):
        pass

    def commit(self):
        pass

    def add_component(self, comp):
        pass

    def remove_component(self, comp):
        pass

    def count_components(self, kind):
        pass

    def select(self, gauge):
        pass

    def clearSelect(self):
        pass

    def fetch(self, uid):
        pass

    def fetchMatch(self, comp):
        pass

    def modify(self, oldc, newc):
        pass

    def current_component(self):
        pass

    def first_component(self):
        pass

    def next_component(self):
        pass


class FileStore(Store):

    def __init__(self, file):
        e1=icalerror_supress("FILE")
        self._ref = icalfileset_new(file)
        icalerror_restore("FILE",e1)

        if self._ref == None or self._ref == 'NULL':
            raise  Store.ConstructorFailedError(file)

    def __del__(self):
        icalfileset_free(self._ref)

    def path(self):
        return icalfileset_path(self._ref)

    def mark(self):
        icalfileset_mark(self._ref)

    def commit(self):
        icalfileset_commit(self._ref)

    def add_component(self, comp):
        if not isinstance(comp,Component):
            raise Store.AddFailedError("Argument is not a component")

        error = icalfileset_add_component(self._ref,comp.ref())

    def remove_component(self, comp):
        if not isinstance(comp,Component):
            raise Store.AddFailedError("Argument is not a component")

        error = icalfileset_remove_component(self._ref,comp.ref())

    def count_components(self, kind):
        _kind = icalcomponent_string_to_kind(kind)

        return icalfileset_count_components(self._ref, _kind)

    def select(self, gauge):
        error = icalfileset_select(self._ref, gauge.ref())

    def clearSelect(self):
        icalfileset_clear(self._ref)

    def fetch(self, uid):
        comp_ref = icalfileset_fetch(self._ref, uid)

        if comp_ref == None:
            return None

        return CloneComponent(comp_ref)

    def fetchMatch(self, comp):
        if not isinstance(comp,Component):
            raise Store.AddFailedError("Argument is not a component")

        comp_ref = icalfileset_fetch_match(self._ref,comp.ref())

        if comp_ref == None:
            return None

        return CloneComponent(comp_ref)

    def modify(self, oldc, newc):
        pass

    def current_component(self):
        comp_ref = icalfileset_get_current_component(self._ref)

        if comp_ref == None:
            return None

        return CloneComponent(comp_ref)

    def first_component(self):
        comp_ref = icalfileset_get_first_component(self._ref)

        if comp_ref == None:
            return None

        return CloneComponent(comp_ref)

    def next_component(self):

        comp_ref = icalfileset_get_next_component(self._ref)

        if comp_ref == None:
            return None

        return CloneComponent(comp_ref)



#!/usr/bin/env python3
#GI_TYPELIB_PATH=$PREFIX/lib/girepository-1.0/ ./attach.py

###############################################################################
#
# Copyright (C) 2015 William Yu <williamyu@gnome.org>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#
###############################################################################

import gi

gi.require_version('ICalGLib', '3.0')

from gi.repository import ICalGLib

dummy_url = "https://people.gnome.org/~engagement/logos/GnomeLogoVertical.svg";
dummy_data = "It's a super large attachment";

attach_url = ICalGLib.Attach.new_from_url(dummy_url);
assert(attach_url.get_is_url() == 1);

retrieved_url = attach_url.get_url();
assert(retrieved_url == dummy_url);

attach_data = ICalGLib.Attach.new_from_data(dummy_data, None, None);
assert(attach_data.get_is_url() == 0);
retrieved_data = attach_data.get_data();
assert(retrieved_data == dummy_data);



#!/usr/bin/env python3
#GI_TYPELIB_PATH=$PREFIX/lib/girepository-1.0/ ./parameter.py

###############################################################################
#
# Copyright (C) 2015 William Yu <williamyu@gnome.org>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#
###############################################################################

import gi

gi.require_version('ICalGLib', '3.0')

from gi.repository import ICalGLib

# Test interaction with gchar
altrep = "This is an altrep";
parameter = ICalGLib.Parameter.new_altrep(altrep);
retrieved_altrep = parameter.get_altrep();
assert(retrieved_altrep == altrep);
another_altrep = "This is an another altrep";
parameter.set_altrep(another_altrep);
retrieved_altrep = parameter.get_altrep();
assert(retrieved_altrep == another_altrep);

# Test interaction with enum
action = ICalGLib.ParameterAction.X;
parameter = ICalGLib.Parameter.new_actionparam(action);
retrieved_action = parameter.get_actionparam();
assert(retrieved_action == action);
action = ICalGLib.ParameterAction.ASK;
parameter.set_actionparam(action);
retrieved_action = parameter.get_actionparam();
assert(retrieved_action == action);

# Test general utilities
kind = ICalGLib.ParameterKind.ACTIONPARAM_PARAMETER;
parameter = ICalGLib.Parameter.new(kind);
assert(parameter.isa() == kind);
assert(parameter.isa_parameter() == 1);
string = parameter.as_ical_string();
assert(string == None);
clone = parameter.clone();
assert(clone.isa() == kind);
assert(clone.isa_parameter() == 1);
string = clone.as_ical_string();
assert(string == None);

string = ICalGLib.Parameter.kind_to_string(kind);
assert(string == "ACTIONPARAM");
assert(ICalGLib.Parameter.kind_from_string(string) == kind);

value = "This is a value";
typevalue = string + "=" + value;
parameter = ICalGLib.Parameter.new_from_string(typevalue);
assert(parameter.as_ical_string() == typevalue);
assert(parameter.isa() == kind);
assert(parameter.isa_parameter() == 1);

another_parameter = ICalGLib.Parameter.new_from_value_string(kind, value);
assert(another_parameter.as_ical_string() == typevalue);



#!/usr/bin/env python3
#GI_TYPELIB_PATH=$PREFIX/lib/girepository-1.0/ ./component.py

###############################################################################
#
# Copyright (C) 2015 William Yu <williamyu@gnome.org>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#
###############################################################################

import gi

gi.require_version('ICalGLib', '3.0')

from gi.repository import ICalGLib

import os
import sys

event_str1 = \
    "BEGIN:VEVENT\n"                                \
    "UID:event-uid-123\n"                           \
    "SUMMARY;LANGUAGE=en-US:test1\n"                \
    "DTSTART;TZID=Europe/Prague:20140306T090000\n"  \
    "DTEND;TZID=Europe/Prague:20140306T093000\n"    \
    "CLASS:PUBLIC\n"                                \
    "PRIORITY:5\n"                                  \
    "DTSTAMP:20140207T020756Z\n"                    \
    "TRANSP:OPAQUE\n"                               \
    "STATUS:CONFIRMED\n"                            \
    "SEQUENCE:0\n"                                  \
    "LOCATION;LANGUAGE=en-US:Location\n"            \
    "END:VEVENT\n"

event_str2 = \
    "BEGIN:VEVENT\n"                                \
    "UID:event-uid-123\n"                           \
    "SUMMARY;LANGUAGE=en-US:test2\n"                \
    "DTSTART;TZID=Europe/Prague:20140306T090000\n"  \
    "DTEND;TZID=Europe/Berlin:20140306T093000\n"    \
    "CLASS:PUBLIC\n"                                \
    "PRIORITY:5\n"                                  \
    "DTSTAMP:20140207T020756Z\n"                    \
    "TRANSP:OPAQUE\n"                               \
    "STATUS:CONFIRMED\n"                            \
    "SEQUENCE:0\n"                                  \
    "LOCATION;LANGUAGE=en-US:Location\n"            \
    "END:VEVENT\n"

event_str3 = \
    "BEGIN:VEVENT\n"                                \
    "UID:event-uid-123\n"                           \
    "SUMMARY;LANGUAGE=en-US:test3\n"                \
    "DTSTART;TZID=Europe/Prague:20140306T090000\n"  \
    "DTEND;TZID=Europe/Berlin:20140306T093000\n"    \
    "CLASS:PUBLIC\n"                                \
    "PRIORITY:5\n"                                  \
    "DTSTAMP:20140207T020756Z\n"                    \
    "TRANSP:OPAQUE\n"                               \
    "STATUS:CONFIRMED\n"                            \
    "SEQUENCE:0\n"                                  \
    "LOCATION;LANGUAGE=en-US:Location\n"            \
    "END:VEVENT\n"

event_str4 = \
    "BEGIN:VEVENT\n"                                \
    "UID:event-uid-123\n"                           \
    "SUMMARY;LANGUAGE=en-US:test4\n"                \
    "DTSTART;TZID=Europe/Prague:20140306T090000\n"  \
    "DTEND;TZID=Europe/Berlin:20140306T093000\n"    \
    "CLASS:PUBLIC\n"                                \
    "PRIORITY:5\n"                                  \
    "DTSTAMP:20140207T020756Z\n"                    \
    "TRANSP:OPAQUE\n"                               \
    "STATUS:CONFIRMED\n"                            \
    "SEQUENCE:0\n"                                  \
    "LOCATION;LANGUAGE=en-US:Location\n"            \
    "END:VCALENDAR\n"

event_str5 = \
    "BEGIN:VCALENDAR\n"                             \
    "UID:event-uid-123\n"                           \
    "SUMMARY;LANGUAGE=en-US:test5\n"                \
    "DTSTART;TZID=Europe/Prague:20140306T090000\n"  \
    "DTEND;TZID=Europe/Berlin:20140306T093000\n"    \
    "CLASS:PUBLIC\n"                                \
    "PRIORITY:5\n"                                  \
    "DTSTAMP:20140207T020756Z\n"                    \
    "TRANSP:OPAQUE\n"                               \
    "STATUS:CONFIRMED\n"                            \
    "SEQUENCE:0\n"                                  \
    "LOCATION;LANGUAGE=en-US:Location\n"            \
    "END:VCALENDAR\n"

recurring_str = \
    "BEGIN:VEVENT\r\n"                              \
    "UID:recurring\r\n"                             \
    "DTSTAMP:20180403T101443Z\r\n"                  \
    "DTSTART:20180320T150000Z\r\n"                  \
    "DTEND:20180320T153000Z\r\n"                    \
    "SUMMARY:Recurring event\r\n"                   \
    "CREATED:20180403T113809Z\r\n"                  \
    "LAST-MODIFIED:20180403T113905Z\r\n"            \
    "RRULE:FREQ=DAILY;COUNT=10;INTERVAL=1\r\n"      \
    "END:VEVENT\r\n"

#############################################################

class TestCounter:
    counter = 0
    def inc(self):
        self.counter = self.counter + 1

def foreachTZIDCb(param, user_data):
    user_data.inc()

def foreachRecurrenceCb(comp, span, user_data):
    user_data.inc()

#############################################################

def main():
    #Test as_ical_string
    comp = ICalGLib.Component.new_from_string(event_str1);
    string = comp.as_ical_string();

    #Test new_clone
    clone = comp.clone();
    string1 = clone.as_ical_string();
    assert(string == string1);
    assert(comp.is_valid() == 1);
    assert(comp.isa_component() == 1);
    assert(comp.isa() == ICalGLib.ComponentKind.VEVENT_COMPONENT);

    #Test check_restrictions
    assert(comp.check_restrictions() == 0);

    #Test count_errors
    assert(comp.count_errors() == 0);

    #Test kind_is_valid
    assert(ICalGLib.Component.kind_is_valid(ICalGLib.ComponentKind.VEVENT_COMPONENT) == True);

    #Test kind_to_string
    kind_string = ICalGLib.Component.kind_to_string(ICalGLib.ComponentKind.VEVENT_COMPONENT);
    assert(ICalGLib.Component.kind_from_string(kind_string) == ICalGLib.ComponentKind.VEVENT_COMPONENT);

    #Test child component manipulation
    parent = ICalGLib.Component.new_from_string(event_str1);
    comp1 = ICalGLib.Component.new_from_string(event_str2);
    comp2 = ICalGLib.Component.new_from_string(event_str3);
    comp3 = ICalGLib.Component.new_from_string(event_str4);
    comp4 = ICalGLib.Component.new_from_string(event_str5);

    parent.add_component(comp1);
    parent.add_component(comp2);
    parent.add_component(comp3);
    parent.add_component(comp4);

    assert parent.count_components(ICalGLib.ComponentKind.VEVENT_COMPONENT) == 3;
    assert parent.count_components(ICalGLib.ComponentKind.VCALENDAR_COMPONENT) == 1;

    #Traverse with internal API.
    count = parent.count_components(ICalGLib.ComponentKind.VEVENT_COMPONENT);
    child_component = parent.get_first_component(ICalGLib.ComponentKind.VEVENT_COMPONENT);
    for i in range(0, count):
        prefix = "test"
        index = i+2;
        assert(child_component.get_summary() == prefix + str(index));
        if (i != count-1):
            child_component = parent.get_next_component(ICalGLib.ComponentKind.VEVENT_COMPONENT);

    #Traverse with external API.
    iter = parent.begin_component(ICalGLib.ComponentKind.VEVENT_COMPONENT);
    child_component = iter.deref();
    for i in range(0, count):
        prefix = "test"
        index = i+2;
        assert(child_component.get_summary() == prefix + str(index));
        if (i != count-1):
            child_component = iter.next();

    iter = parent.end_component(ICalGLib.ComponentKind.VEVENT_COMPONENT);
    child_component = iter.prior();
    for i in range(0, count):
        prefix = "test"
        index = count + 1 - i;
        assert(child_component.get_summary() == prefix + str(index));
        if (i != count - 1):
            child_component = iter.prior();

    #Traverse and remove with external API.
    iter = parent.begin_component(ICalGLib.ComponentKind.VEVENT_COMPONENT);
    child_component = iter.deref();
    for i in range(0, count):
        if (i != count - 1):
            iter.next();
        parent.remove_component(child_component);
        if (i != count - 1):
            child_component = iter.deref();
    assert parent.count_components(ICalGLib.ComponentKind.VEVENT_COMPONENT) == 0;

    #Test property mainpulation
    property_string = "SUMMARY:Bastille Day Party";
    string_property = ICalGLib.Property.new_from_string(property_string);
    component = ICalGLib.Component.new(ICalGLib.ComponentKind.VEVENT_COMPONENT);
    component.add_property(string_property);
    assert(component.count_properties(ICalGLib.PropertyKind.SUMMARY_PROPERTY) == 1);
    component.remove_property(string_property);
    assert(component.count_properties(ICalGLib.PropertyKind.SUMMARY_PROPERTY) == 0);

    component.add_property(string_property);
    property_string2 = "SUMMARY:event-uid-123";
    string_property2 = ICalGLib.Property.new_from_string(property_string2);
    component.add_property(string_property2);
    component.add_property(ICalGLib.Property.new_from_string("SUMMARY:20140306T090000"));
    assert(component.count_properties(ICalGLib.PropertyKind.SUMMARY_PROPERTY) == 3);
    property1 = component.get_first_property(ICalGLib.PropertyKind.SUMMARY_PROPERTY);
    assert(property1.as_ical_string().split('\n', 1)[0] == "SUMMARY:Bastille Day Party\r");
    property2 = component.get_next_property(ICalGLib.PropertyKind.SUMMARY_PROPERTY);
    assert(property2.as_ical_string().split('\n', 1)[0] == "SUMMARY:event-uid-123\r");
    property3 = component.get_next_property(ICalGLib.PropertyKind.SUMMARY_PROPERTY);
    assert(property3.as_ical_string().split('\n', 1)[0] == "SUMMARY:20140306T090000\r");

    #Test getters and setters
    #Test get_dtstart and get_dtend
    comp = ICalGLib.Component.new_from_string(event_str1);
    dtstart = comp.get_dtstart();
    start_string = ICalGLib.Time.as_ical_string(dtstart);
    assert(start_string == "20140306T090000");
    dtend = comp.get_dtend();
    end_string = dtend.as_ical_string();
    assert(end_string == "20140306T093000");

    #Test span
    span = comp.get_span();
    assert(span.get_start() == 1394096400);
    assert(span.get_end() == 1394098200);
    assert(span.get_is_busy() == 1);
    utc = ICalGLib.Timezone.get_utc_timezone();
    comp.set_dtstart(ICalGLib.Time.new_from_timet_with_zone(1494096400, 0, utc));
    comp.set_dtend(ICalGLib.Time.new_from_timet_with_zone(1494098200, 0, utc));
    span = comp.get_span();
    assert(span.get_start() == 1494096400);
    assert(span.get_end() == 1494098200);
    assert(span.get_is_busy() == 1);

    #Test set_summary/get_summary
    assert(comp.get_summary() == "test1");
    comp.set_summary("newSummary");
    assert(comp.get_summary() == "newSummary");

    #Test set_comment/get_comment
    assert(comp.get_comment() == None);
    comp.set_comment("newcomment");
    assert(comp.get_comment() == "newcomment");

    #Test set_uid/get_uid
    assert(comp.get_uid() == "event-uid-123");
    comp.set_uid("newuid");
    assert(comp.get_uid() == "newuid");

    #Test set_relcalid/get_relcalid
    assert(comp.get_relcalid() == None);
    comp.set_relcalid("newrelcalid");
    assert(comp.get_relcalid() == "newrelcalid");

    #Test set_description/get_description
    assert(comp.get_description() == None);
    comp.set_description("newdescription");
    assert(comp.get_description() == "newdescription");

    #Test set_location/get_location
    assert(comp.get_location() == "Location");
    comp.set_location("newlocation");
    assert(comp.get_location() == "newlocation");

    #Test set_sequence/get_sequence
    assert(comp.get_sequence() == 0);
    comp.set_sequence(5);
    assert(comp.get_sequence() == 5);

    #Call comp_foreach_tzid
    comp = ICalGLib.Component.new_from_string(event_str1);
    counter = TestCounter()
    comp.foreach_tzid(foreachTZIDCb, counter);
    assert counter.counter == 2

    counter = TestCounter()
    comp = ICalGLib.Component.new_from_string(recurring_str)
    comp.foreach_recurrence(ICalGLib.Time.new_from_string("20180321T000000Z"), ICalGLib.Time.new_from_string("20180323T235959Z"), foreachRecurrenceCb, counter)
    assert counter.counter == 3

    comp = ICalGLib.Component.new_from_string(event_str1);
    prop = comp.get_first_property(ICalGLib.PropertyKind.DTSTART_PROPERTY)
    prop.remove_parameter_by_kind(ICalGLib.ParameterKind.TZID_PARAMETER)
    tz = ICalGLib.Timezone.get_builtin_timezone("Europe/Prague")

    ICalGLib.Timezone.set_tzid_prefix(tz.get_tzid().replace("Europe/Prague", ""))

    prop.set_parameter(ICalGLib.Parameter.new_tzid(tz.get_tzid()))

    itt = prop.get_datetime_with_component(comp)
    assert itt.get_timezone() != None
    assert itt.get_timezone().get_location() == "Europe/Prague"

    itt = prop.get_datetime_with_component(None)
    assert itt.get_timezone() != None
    assert itt.get_timezone().get_location() == "Europe/Prague"

    itt = comp.get_dtstart()
    assert itt.get_timezone() != None
    assert itt.get_timezone().get_location() == "Europe/Prague"

if __name__ == "__main__":
    main()



#======================================================================
# FILE: Error.py
# CREATOR: eric
#
# (C) COPYRIGHT 2001, Eric Busboom <eric@civicknowledge.com>
# (C) COPYRIGHT 2001, Patrick Lewis <plewis@inetarena.com>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/


# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#    The LGPL as published by the Free Software Foundation, version
#    2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.html
#
#  Or:
#
#    The Mozilla Public License Version 2.0. You may obtain a copy of
#    the License at https://www.mozilla.org/MPL/
#===========================================================

class LibicalError(Exception):
    "Libical Error"

    def __init__(self,str):
        Exception.__init__(self,str)

    def __str__(self):
        return Exception.__str__(self)+"\nLibical errno: "+icalerror_perror()



#!/usr/bin/env python3
#GI_TYPELIB_PATH=$PREFIX/lib/girepository-1.0/ ./duration-type.py

###############################################################################
#
# Copyright (C) 2015 William Yu <williamyu@gnome.org>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#
###############################################################################

import gi

gi.require_version('ICalGLib', '3.0')

from gi.repository import ICalGLib

length = 1000000000;
bad_string = "This is a bad string";

duration = ICalGLib.Duration.new_from_int(length);
assert(duration.as_int() == length);
length_in_string = duration.as_ical_string();
duration1 = ICalGLib.Duration.new_from_string(length_in_string);
assert(duration1.as_ical_string() == length_in_string);
assert(length == duration1.as_int());

duration = ICalGLib.Duration.new_from_string(bad_string);
duration_bad = ICalGLib.Duration.new_bad_duration();
assert(duration.as_ical_string() == duration_bad.as_ical_string());
assert(duration.is_bad_duration() == 1);
duration_null = ICalGLib.Duration.new_null_duration();
assert(duration_null.is_null_duration() == 1);



#!/usr/bin/env python
#======================================================================
# FILE: DerivedProperties.py
# CREATOR: eric
#
# (C) COPYRIGHT 2001, Eric Busboom <eric@civicknowledge.com>
# (C) COPYRIGHT 2001, Patrick Lewis <plewis@inetarena.com>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#======================================================================

from Property import Property
from Time import Time
from Period import Period
from Duration import Duration

def RDate(arg):

    class RDate_Time(Time):
        def __init__(self,arg): Time.__init__(self,arg,"RDATE")

    class RDate_Period(Period):
        def __init__(self,arg): Period.__init__(self,arg,"RDATE")

    p = None
    for c in [RDate_Time, RDate_Period]:
        try: return c(arg)
        except Property.ConstructorFailedError, d: pass
    raise Property.ConstructorFailedError("Failed to construct RDATE from "+str(arg))


def Trigger(arg):
    class Trigger_Time(Time):
        def __init__(self,arg): Time.__init__(self,arg,"TRIGGER")

    class Trigger_Duration(Duration):
        def __init__(self,arg): Duration.__init__(self,arg,"TRIGGER")

    p = None
    for c in [Trigger_Duration, Trigger_Time]:
        try: return c(arg)
        except Property.ConstructorFailedError, d: pass
    raise Property.ConstructorFailedError("Failed to construct TRIGGER from "+str(arg))



class Recurrence_Id(Time):
    """Class for RECURRENCE-ID property.

    Usage:
    Reccurence_Id(dict)         # A normal property dictionary
    Reccurence_Id("19960401")   # An iCalendar string
    Reccurence_Id(8349873494)   # Seconds from epoch

    If the 'dict' constructor is used, 'name' and 'value_type'
    entries in dict are ignored and automatically set with the appropriate
    values.
    """

    def __init__(self, dict={}):
        Time.__init__(self, dict)
        Property.name(self, 'RECURRENCE-ID')

    def name(self):
        return Property.name(self)

    def _doParam(self, parameter, v):
        if v!=None:
            self[parameter]=v
        return self[parameter]

    # Enumerated parameters
    def value_parameter(self, v=None):
        """Sets or gets the VALUE parameter value.

        The value passed should be either "DATE-TIME" or "DATE".  Setting this
        parameter has no impact on the property's value_type.  Doing something
        like:

        rid=Recurrence_Id("19960401")    # Sets value & makes value_type="DATE"
        rid.value_parameter("DATE-TIME") # Sets the parameter VALUE=DATE-TIME

        Would be allowed (even though it is wrong), so pay attention.
        Verifying the component will reveal the error.
        """
        if v!=None and v!="DATE" and v!="DATE-TIME":
            raise ValueError, "%s is an invalid VALUE parameter value" % str(v)
        self._doParam("VALUE", v)

    def tzid(self, v=None):
        "Sets or gets the TZID parameter value."
        self._doParam("TZID", v)

    def range_parameter(self, v=None): # 'range' is a builtin function
        "Sets or gets the RANGE parameter value."
        if v!=None and v!="THISANDPRIOR" and v!= "THISANDFUTURE":
            raise ValueError, "%s is an invalid RANGE parameter value" % str(v)
        self._doParam("RANGE", v)

class Attach(Property):
    """A class representing an ATTACH property.

    Usage:
    Attach(uriString [, parameter_dict])
    Attach(fileObj [, parameter_dict])
    """

    def __init__(self, value=None, parameter_dict={}):
        Property.__init__(self, parameter_dict)
        Property.name(self, 'ATTACH')
        self.value(value)

    def value(self, v=None):
        "Returns or sets the value of the property."
        if v != None:
            if isinstance(v, StringType):  # Is a URI
                self._desc['value']=v
                Property.value_type(self, 'URI')
            else:
                try:
                    tempStr = v.read()
                except:
                    raise TypeError,"%s must be a URL string or file-ish type"\
                          % str(v)
                self._desc['value'] = base64.encodestring(tempStr)
                Property.value_type(self, 'BINARY')
        else:
            return self._desc['value']

    def name(self):
        "Returns the name of the property."
        return Property.name(self)

    def value_type(self):
        return Property.value_type(self)

    def fmttype(self, v=None):
        "Gets or sets the FMTYPE parameter."
        if v!= None:
            self['FMTTYPE']=v
        else:
            return self['FMTTYPE']



#!/usr/bin/env python
#======================================================================
# FILE: test.py
# CREATOR: eric
#
# (C) COPYRIGHT 2001, Eric Busboom <eric@civicknowledge.com>
# (C) COPYRIGHT 2001, Patrick Lewis <plewis@inetarena.com>
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of either:
#
#   The LGPL as published by the Free Software Foundation, version
#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
#
# Or:
#
#   The Mozilla Public License Version 2.0. You may obtain a copy of
#   the License at https://www.mozilla.org/MPL/
#======================================================================

import LibicalWrap
from Libical import *

def error_type():
    error = icalerror_perror()
    return error[:index(error,':')]

comp_str = """
BEGIN:VCALENDAR
VERSION:2.0
PRODID:-//ABC Corporation//NONSGML My Product//EN
METHOD:REQUEST
BEGIN:VEVENT
ATTENDEE;RSVP=TRUE;ROLE=REQ-PARTICIPANT;CUTYPE=GROUP:MAILTO:employee-A@host.com
COMMENT: When in the course of writing comments and nonsense text\, it
 becomes necessary to insert a newline
DTSTART:19972512T120000
DTSTART:19970101T120000Z
DTSTART:19970101
DURATION:P3DT4H25M
FREEBUSY:19970101T120000/19970101T120000
FREEBUSY:19970101T120000/PT3H
FREEBUSY:19970101T120000/PT3H
END:VEVENT
END:VCALENDAR"""


def test_property():

    print "--------------------------- Test Property ----------------------"


    liw = LibicalWrap
    icalprop = liw.icalproperty_new_from_string("ATTENDEE;RSVP=TRUE;ROLE=REQ-PARTICIPANT;CUTYPE=GROUP:MAILTO:employee-A@host.com")

    print liw.icalproperty_as_ical_string(icalprop)

    p = Property(ref=icalprop)

    print p.name()
    print
    print "Parameters:"
    for param in p.parameters():
            print "  ", param, " = ", p[param]
    print
    print p['ROLE']

    p['ROLE'] = 'INDIVIDUAL'

    print p['ROLE']

    p['ROLE'] = 'GROFROMBLATZ'

    print p['ROLE']

    print

    p['X-MAN-FAVOURITE'] = 'Wolverine'
    p['X-FILES-FAVOURITE'] = 'Mulder'

    print p['X-MAN-FAVOURITE']

    assert(p['X-MAN-FAVOURITE'] == 'Wolverine')
    assert(p['X-FILES-FAVOURITE'] == 'Mulder')
    assert(p['X-FILES-FAVOURITE'] != 'Scully')

    print p.value()
    p.value("mailto:Bob@bob.com")
    print p.value()


    print p.as_ical_string()
    del p['ROLE']
    del p['X-MAN-FAVOURITE']

    print p.as_ical_string()


    try:
        p = Property()
    except Property.ConstructorFailedError:
        pass
    else:
        assert(0)

    # X Property
    p = Property("X-COMMENT")

    p.value("This is a sentence, with punctuation; indeed: it is")
    print p

    p.value("This is not approved by the Ministry of Silly Walks")
    print p


    assert(test_enum('METHOD','PUBLISH'))
    assert(not test_enum('METHOD','FOO'))

    assert(test_enum('ACTION','AUDIO'))
    assert(not test_enum('ACTION','OPAQUE'))

def test_time():
    "Test routine"

    print"-------------------Test Time  --------------------------------"

    t = Time("19970325T123010Z",'DTSTART')

    assert(t.year == 1997)
    assert(t.month == 3)
    assert(t.day == 25)
    assert(t.hour == 12)
    assert(t.minute == 30)
    assert(t.second == 10)
    assert(t.is_utc())
    assert(not t.is_date())

    print t

    t.timezone("America/Los_Angeles")
    print str(t)
    print t.timezone()
    #assert(str(t)=='DTSTART;TZID=America/Los_Angeles:19970325T123010')
    assert(str(t)=='DTSTART;TZID=/freeassociation.sourceforge.net/Tzfile/America/Los_Angeles:19970325T053010')

    t.second = t.second+80

    t.timezone("UTC")
    print t.minute, t.second
    assert(t.minute == 31)
    assert(t.second == 30)

    d = Duration(3600,"DURATION")
    t2 = t + d

    print t2
    assert(t2.hour == 13)

    t2 = t - d

    print t2
    assert(isinstance(t2,Time))
    assert(t2.hour == 11)

    # test int args
    t = Time(2)
    print t

    # test float args
    t = Time(2.5)
    print t

def test_period():

    print"-------------------Test Period--------------------------------"

    p = Period("19970101T180000Z/19970101T233000Z")

    print p

    assert(str(p) == 'FREEBUSY:19970101T180000Z/19970101T233000Z')

    print p.start()
    assert(str(p.start()) == 'DTSTART:19970101T180000Z')

    print p.end()
    assert(str(p.end()) == 'DTEND:19970101T233000Z')

    print p.duration()
    assert(str(p.duration()) == 'DURATION:PT5H30M')
    p = None

    p = Period("19970101T180000Z/PT5H30M")
    print p

    print p.start()
    assert(str(p.start()) == 'DTSTART:19970101T180000Z')

    print p.end()
    assert(str(p.end()) == 'DTEND:19970101T233000Z')

    print p.duration()
    assert(str(p.duration()) == 'DURATION:PT5H30M')


def test_duration():

    print "-------------- Test Duration ----------------"

    # Ical string

    d = Duration("P3DT4H25M")

    print str(d)

    assert(str(d) == "DURATION:P3DT4H25M")

    print d.seconds()

    assert(d.seconds() == 275100)

    # seconds

    d = Duration(-275100)

    print str(d)

    assert(str(d) == "DURATION:-P3DT4H25M")

    print d.seconds()

    assert(d.seconds() == -275100)

    #error

    try:
        d = Duration("P10WT7M")
        print str(d)
        assert(0)
    except: pass

    try:
        d = Duration("Pgiberish")
        print str(d)
        assert(0)
    except:
        pass



def test_attach():

    file = open('littlefile.txt')
    attachProp = Attach(file)
    file.close()
    attachProp.fmttype('text/ascii')
    print "\n" + attachProp.name()
    print attachProp.value_type()
    print attachProp.fmttype()
    attachProp['fmttype']=None
    print "Calling value()"
    print attachProp.value()
    print "Calling asIcalString()"
    print attachProp.as_ical_string()


def test_component():

    print "------------------- Test Component ----------------------"


    c = NewComponent(comp_str);

    props = c.properties()

    for p in props:
        print p.as_ical_string()

    inner = c.components()[0]

    print inner
    print type(inner)


    props = inner.properties()

    for p in props:
        print p.as_ical_string()

    dtstart = inner.properties('DTSTART')[0]

    print dtstart

    print "\n Orig hour: ", dtstart.hour
    assert(dtstart.hour == 12)

    dtstart.hour = dtstart.hour + 5

    print "\n New hour: ", dtstart.hour
    assert(dtstart.hour == 17)

    attendee = inner.properties('ATTENDEE')[0]

    print attendee

    t = Time("20011111T123030")
    t.name('DTEND')

    inner.add_property(t)


    print c

    dtstart1 = inner.properties('DTSTART')[0]
    dtstart2 = inner.properties('DTSTART')[0]
    dtstart3 = inner.property('DTSTART')

    assert(dtstart1 is dtstart2)
    assert(dtstart1 == dtstart2)

    assert(dtstart1 is dtstart3)
    assert(dtstart1 == dtstart3)


    p = Property(type="SUMMARY");
    p.value("This is a summary")

    inner.properties().append(p)

    print inner.as_ical_string()

    p = inner.properties("SUMMARY")[0]
    assert(p!=None);
    print str(p)
    assert(str(p) == "SUMMARY:This is a summary")

    inner.properties()[:] = [p]

    print inner.as_ical_string()

    # test sequence
    event = Event()

    try:
       event.sequence("foo")
    except TypeError:
       pass

    event.sequence(-1)
    print event.sequence()

    event.sequence(1)
    event.sequence(88)
    print event.sequence()

def test_event():
    print "------------ Event Class ----------------------"

    event = Event()

    event.method('REQUEST')
    event.version('2.0')

    event.created("20010313T123000Z")
    print "created =", event.created()
    assert (event.created() == Time("20010313T123000Z"))

    event.organizer("MAILTO:j_doe@nowhere.com")
    org = event.organizer()
    print org.cn()
    org.cn('Jane Doe')
    assert (isinstance(org, Organizer))
    print "organizer =", event.organizer()
    assert (event.organizer().value() == "MAILTO:j_doe@nowhere.com")

    event.dtstart("20010401T183000Z")
    print "dtstart =", event.dtstart()
    assert (event.dtstart()== Time("20010401T183000Z"))

    dtend = Time('20010401T190000Z', 'DTEND')
    event.dtend(dtend)
    assert (event.dtend() ==dtend )
    assert (event.dtend() == Time('20010401T190000Z'))

    att = Attendee()
    att.value('jsmith@nothere.com')
    event.attendees(('ef_hutton@listenup.com', att))

    event.x_properties('X-TEST',('foo', 'bar'))
    event.x_properties('X-TEST2',('foo, biz', 'bar, biz'))

    inner = event.components()[0]
    for e in inner.properties('X-TEST'):
        print " ", e.as_ical_string()

    assert(len(event.x_properties('X-TEST'))==2)

    event.description("A short description.  Longer ones break things. Really. What does it break. The code is supposed to handle really long lines, longer, in fact, than any sane person would create except by writing a random text generator or by excerpting text from a less sane person. Actually, it did \"break\" and I had to remove an \n assert to fix it.")
    event.status('TeNtAtIvE')

    print event.as_ical_string()


def test_derivedprop():

    print "------------ Derived Properties -----------------"

    p = RDate("20011111T123030")

    print str(p)


    p = RDate("19970101T120000/19970101T123000")

    print str(p)

    try:
        p = RDate("P3DT4H25M")
        print str(p)
        assert(0)
    except: pass


    p = Trigger("P3DT4H25M")

    print str(p)

    p = Trigger("20011111T123030")

    print str(p)

    try:
        p = Trigger("19970101T120000/19970101T123000")
        print str(p)
        assert(0)
    except: pass

def test_gauge():
    print "------------ Gauge -----------------"
    event = Event()

    event.method('REQUEST')
    event.version('2.0')
    event.created("20010313T123000Z")
    event.organizer("MAILTO:j_doe@nowhere.com")
    org = event.organizer()
    org.cn('Jane Doe')
    event.dtstart("20010401T183000Z")
    dtend = Time('20010401T190000Z', 'DTEND')
    event.dtend(dtend)
    event.description("A short description.")
    event.status('TeNtAtIvE')

    print event.as_ical_string()

    gauge = Gauge(sql="SELECT * FROM VEVENT WHERE DTSTART > '20010401T180000Z'")

    assert(gauge.compare(event) == 1)

    gauge = Gauge(sql="SELECT * FROM VEVENT WHERE DTSTART > '20010401T190000Z'")

    assert(gauge.compare(event) == 0)

def do_test_store(storeobj=None, *args):
    assert(storeobj != None)
    store = storeobj(*args)
    assert(store != None)

    print ">------------ ",
    print store.__class__,
    print "Store -----------------"


    # create fileset

    event = Event()

    event.method('REQUEST')
    event.version('2.0')
    event.created("20010313T123000Z")
    event.organizer("MAILTO:j_doe@nowhere.com")
    event.dtstart("20010401T183000Z")
    event.duration('PT3H')

    event.description("A short description.")

    # for i = 1 to 10
    #  copy event
    #  munge uid and increment month
    for i in range(1,11):
        newevent = event.clone()
        newevent.uid("%d@localhost" % (i,))
        newevent.dtstart().month = newevent.dtstart().month + i

        #print ne
        store.add_component(newevent)

    # commit
    store.commit()
    assert(store.count_components("VCALENDAR") == 10)
    # free
    del(store)

    # open again
    store = storeobj(*args)
    # assert count of components = 10
    assert(store.count_components("VCALENDAR") == 10)

    # print them out
    # fetch by uid
    n7 = store.fetch("7@localhost")
    print n7
    # fetch by match

    n7m = store.fetchMatch(n7)
    assert(str(n7) == str(n7m))

    # modify in memory
    n7.uid("42@localhost")
    del(store)
    del(n7)

    store = storeobj(*args)
    assert(store.fetch("42@localhost") == None)
    n7 = store.fetch("7@localhost")
    n7.uid("42@localhost")
    store.mark()
    store.commit()
    del(store)
    store = storeobj(*args)
    assert(store.fetch("7@localhost") == None)

    # fetch by gauge

    gauge = Gauge(sql="SELECT * FROM VEVENT WHERE DTSTART > '20010601T000000Z' AND DTSTART < '20010901T000000Z'")

    store.select(gauge)

    count = 0

    c = store.first_component()
    while c != None:
            print c.uid()
            print c.dtstart()
            print
            count = count + 1
            c = store.next_component()

    store.clearSelect()

    assert(count == 3)

    # remove all of them
    c = store.first_component()
    while c != None:
            print c.uid()
            store.remove_component(c)
            c = store.first_component()

    assert(store.count_components("VCALENDAR") == 0)
    store.commit()
    assert(store.count_components("VCALENDAR") == 0)
    # print them out
    # assert count of components = 0


def test_store():
    print "------------ Store -----------------"
    do_test_store(FileStore,"filesetout.ics")

def run_tests():
    print "Running unit tests for:", ICAL_PACKAGE, ICAL_VERSION

    test_property()

    test_time()

    test_period()

    test_component()

    test_duration()

    test_derivedprop()

    test_event()

    #test_attach()

    test_gauge()

    test_store()




if __name__ == "__main__":
    run_tests()
